diff -pruN 0.7.2-1/autogen.sh 0.7.3-1/autogen.sh
--- 0.7.2-1/autogen.sh	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/autogen.sh	2020-10-07 16:30:24.000000000 +0000
@@ -3,7 +3,7 @@
 if [ -d .git ]; then
   git describe --always --tags --dirty > version # get version from git
 else
-  echo 0.7.2 > version # hard coded versions
+  echo 0.7.3 > version # hard coded versions
 fi
 
 libtoolize
diff -pruN 0.7.2-1/cava.c 0.7.3-1/cava.c
--- 0.7.2-1/cava.c	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/cava.c	2020-10-07 16:30:24.000000000 +0000
@@ -8,6 +8,11 @@
 
 #include <fcntl.h>
 #include <math.h>
+
+#ifndef M_PI
+#define M_PI 3.1415926535897932385
+#endif
+
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -57,19 +62,16 @@
 #define GCC_UNUSED /* nothing */
 #endif
 
-#define LEFT_CHANNEL 1
-#define RIGHT_CHANNEL 2
-
-// struct termios oldtio, newtio;
-// int M = 8 * 1024;
-
+pthread_mutex_t lock;
 // used by sig handler
-// needs to know output mode in orer to clean up terminal
+// needs to know output mode in order to clean up terminal
 int output_mode;
 // whether we should reload the config or not
 int should_reload = 0;
 // whether we should only reload colors or not
 int reload_colors = 0;
+// whether we should quit
+int should_quit = 0;
 
 // these variables are used only in main, but making them global
 // will allow us to not free them on exit without ASan complaining
@@ -132,57 +134,6 @@ static bool directory_exists(const char
 
 #endif
 
-int *separate_freq_bands(int FFTbassbufferSize, fftw_complex out_bass[FFTbassbufferSize / 2 + 1],
-                         int FFTmidbufferSize, fftw_complex out_mid[FFTmidbufferSize / 2 + 1],
-                         int FFTtreblebufferSize,
-                         fftw_complex out_treble[FFTtreblebufferSize / 2 + 1], int bass_cut_off_bar,
-                         int treble_cut_off_bar, int number_of_bars,
-                         int FFTbuffer_lower_cut_off[256], int FFTbuffer_upper_cut_off[256],
-                         double eq[256], int channel, double sens, double ignore) {
-    int n, i;
-    double peak[257];
-    static int bars_left[256];
-    static int bars_right[256];
-    double y[FFTbassbufferSize / 2 + 1];
-    double temp;
-
-    // process: separate frequency bands
-    for (n = 0; n < number_of_bars; n++) {
-
-        peak[n] = 0;
-        i = 0;
-
-        // process: get peaks
-        for (i = FFTbuffer_lower_cut_off[n]; i <= FFTbuffer_upper_cut_off[n]; i++) {
-            if (n <= bass_cut_off_bar) {
-                y[i] = hypot(out_bass[i][0], out_bass[i][1]);
-            } else if (n > bass_cut_off_bar && n <= treble_cut_off_bar) {
-                y[i] = hypot(out_mid[i][0], out_mid[i][1]);
-            } else if (n > treble_cut_off_bar) {
-                y[i] = hypot(out_treble[i][0], out_treble[i][1]);
-            }
-
-            peak[n] += y[i]; // adding upp band
-        }
-
-        peak[n] = peak[n] /
-                  (FFTbuffer_upper_cut_off[n] - FFTbuffer_lower_cut_off[n] + 1); // getting average
-        temp = peak[n] * sens * eq[n]; // multiplying with k and sens
-        // printf("%d peak o: %f * sens: %f * k: %f = f: %f\n", o, peak[o], sens, eq[o], temp);
-        if (temp <= ignore)
-            temp = 0;
-        if (channel == LEFT_CHANNEL)
-            bars_left[n] = temp;
-        else
-            bars_right[n] = temp;
-    }
-
-    if (channel == LEFT_CHANNEL)
-        return bars_left;
-    else
-        return bars_right;
-}
-
 int *monstercat_filter(int *bars, int number_of_bars, int waves, double monstercat) {
 
     int z;
@@ -227,15 +178,18 @@ int main(int argc, char **argv) {
     pthread_t p_thread;
     int thr_id GCC_UNUSED;
     float cut_off_frequency[256];
+    float upper_cut_off_frequency[256];
     float relative_cut_off[256];
+    double center_frequencies[256];
     int bars[256], FFTbuffer_lower_cut_off[256], FFTbuffer_upper_cut_off[256];
-    int *bars_left, *bars_right, *bars_mono;
+    int *bars_left, *bars_right;
+    double *temp_l, *temp_r;
     int bars_mem[256];
     int bars_last[256];
     int previous_frame[256];
-    int sleep = 0;
+    int sleep_counter = 0;
     int n, height, lines, width, c, rest, inAtty, fp, fptest, rc;
-    bool silence;
+    bool silence = false;
     // int cont = 1;
     int fall[256];
     // float temp;
@@ -243,7 +197,7 @@ int main(int argc, char **argv) {
     double eq[256];
     float g;
     struct timespec req = {.tv_sec = 0, .tv_nsec = 0};
-    struct timespec sleep_mode_timer = {.tv_sec = 0, .tv_nsec = 0};
+    struct timespec sleep_mode_timer = {.tv_sec = 1, .tv_nsec = 0};
     char configPath[PATH_MAX];
     char *usage = "\n\
 Usage : " PACKAGE " [options]\n\
@@ -268,7 +222,6 @@ as of 0.4.0 all options are specified in
 
     char ch = '\0';
     int number_of_bars = 25;
-    int sourceIsAuto = 1;
     double userEQ_keys_to_bars_ratio;
 
     struct audio_data audio;
@@ -350,7 +303,7 @@ as of 0.4.0 all options are specified in
             // must set LC_ALL instead.
             // Attempting to set to en_US if not set, if that lang
             // is not installed and LANG is not set there will be
-            // no output, for mor info see #109 #344
+            // no output, for more info see #109 #344
             if (!getenv("LANG"))
                 setlocale(LC_ALL, "en_US.utf8");
             else
@@ -359,7 +312,7 @@ as of 0.4.0 all options are specified in
 
         // input: init
         int bass_cut_off = 150;
-        int treble_cut_off = 1500;
+        int treble_cut_off = 2500;
 
         audio.source = malloc(1 + strlen(p.audio_source));
         strcpy(audio.source, p.audio_source);
@@ -367,8 +320,8 @@ as of 0.4.0 all options are specified in
         audio.format = -1;
         audio.rate = 0;
         audio.FFTbassbufferSize = 4096;
-        audio.FFTmidbufferSize = 1024;
-        audio.FFTtreblebufferSize = 512;
+        audio.FFTmidbufferSize = 2048;
+        audio.FFTtreblebufferSize = 1024;
         audio.terminate = 0;
         if (p.stereo)
             audio.channels = 2;
@@ -386,19 +339,39 @@ as of 0.4.0 all options are specified in
         audio.bass_index = 0;
         audio.mid_index = 0;
         audio.treble_index = 0;
-
+        audio.bass_multiplier = (double *)malloc(audio.FFTbassbufferSize * sizeof(double));
+        audio.mid_multiplier = (double *)malloc(audio.FFTmidbufferSize * sizeof(double));
+        audio.treble_multiplier = (double *)malloc(audio.FFTtreblebufferSize * sizeof(double));
+
+        temp_l = (double *)malloc((audio.FFTbassbufferSize / 2 + 1) * sizeof(double));
+        temp_r = (double *)malloc((audio.FFTbassbufferSize / 2 + 1) * sizeof(double));
+
+        bars_left = (int *)malloc(256 * sizeof(int));
+        bars_right = (int *)malloc(256 * sizeof(int));
+
+        for (int i = 0; i < audio.FFTbassbufferSize; i++) {
+            audio.bass_multiplier[i] =
+                0.5 * (1 - cos(2 * M_PI * i / (audio.FFTbassbufferSize - 1)));
+        }
+        for (int i = 0; i < audio.FFTmidbufferSize; i++) {
+            audio.mid_multiplier[i] = 0.5 * (1 - cos(2 * M_PI * i / (audio.FFTmidbufferSize - 1)));
+        }
+        for (int i = 0; i < audio.FFTtreblebufferSize; i++) {
+            audio.treble_multiplier[i] =
+                0.5 * (1 - cos(2 * M_PI * i / (audio.FFTtreblebufferSize - 1)));
+        }
         // BASS
         // audio.FFTbassbufferSize =  audio.rate / 20; // audio.FFTbassbufferSize;
 
-        audio.in_bass_r = fftw_alloc_real(2 * (audio.FFTbassbufferSize / 2 + 1));
-        audio.in_bass_l = fftw_alloc_real(2 * (audio.FFTbassbufferSize / 2 + 1));
-        memset(audio.in_bass_r, 0, 2 * (audio.FFTbassbufferSize / 2 + 1) * sizeof(double));
-        memset(audio.in_bass_l, 0, 2 * (audio.FFTbassbufferSize / 2 + 1) * sizeof(double));
-
-        out_bass_l = fftw_alloc_complex(2 * (audio.FFTbassbufferSize / 2 + 1));
-        out_bass_r = fftw_alloc_complex(2 * (audio.FFTbassbufferSize / 2 + 1));
-        memset(out_bass_l, 0, 2 * (audio.FFTbassbufferSize / 2 + 1) * sizeof(fftw_complex));
-        memset(out_bass_r, 0, 2 * (audio.FFTbassbufferSize / 2 + 1) * sizeof(fftw_complex));
+        audio.in_bass_r = fftw_alloc_real(audio.FFTbassbufferSize);
+        audio.in_bass_l = fftw_alloc_real(audio.FFTbassbufferSize);
+        audio.in_bass_r_raw = fftw_alloc_real(audio.FFTbassbufferSize);
+        audio.in_bass_l_raw = fftw_alloc_real(audio.FFTbassbufferSize);
+
+        out_bass_l = fftw_alloc_complex(audio.FFTbassbufferSize / 2 + 1);
+        out_bass_r = fftw_alloc_complex(audio.FFTbassbufferSize / 2 + 1);
+        memset(out_bass_l, 0, (audio.FFTbassbufferSize / 2 + 1) * sizeof(fftw_complex));
+        memset(out_bass_r, 0, (audio.FFTbassbufferSize / 2 + 1) * sizeof(fftw_complex));
 
         p_bass_l = fftw_plan_dft_r2c_1d(audio.FFTbassbufferSize, audio.in_bass_l, out_bass_l,
                                         FFTW_MEASURE);
@@ -407,15 +380,15 @@ as of 0.4.0 all options are specified in
 
         // MID
         // audio.FFTmidbufferSize =  audio.rate / bass_cut_off; // audio.FFTbassbufferSize;
-        audio.in_mid_r = fftw_alloc_real(2 * (audio.FFTmidbufferSize / 2 + 1));
-        audio.in_mid_l = fftw_alloc_real(2 * (audio.FFTmidbufferSize / 2 + 1));
-        memset(audio.in_mid_r, 0, 2 * (audio.FFTmidbufferSize / 2 + 1) * sizeof(double));
-        memset(audio.in_mid_l, 0, 2 * (audio.FFTmidbufferSize / 2 + 1) * sizeof(double));
-
-        out_mid_l = fftw_alloc_complex(2 * (audio.FFTmidbufferSize / 2 + 1));
-        out_mid_r = fftw_alloc_complex(2 * (audio.FFTmidbufferSize / 2 + 1));
-        memset(out_mid_l, 0, 2 * (audio.FFTmidbufferSize / 2 + 1) * sizeof(fftw_complex));
-        memset(out_mid_r, 0, 2 * (audio.FFTmidbufferSize / 2 + 1) * sizeof(fftw_complex));
+        audio.in_mid_r = fftw_alloc_real(audio.FFTmidbufferSize);
+        audio.in_mid_l = fftw_alloc_real(audio.FFTmidbufferSize);
+        audio.in_mid_r_raw = fftw_alloc_real(audio.FFTmidbufferSize);
+        audio.in_mid_l_raw = fftw_alloc_real(audio.FFTmidbufferSize);
+
+        out_mid_l = fftw_alloc_complex(audio.FFTmidbufferSize / 2 + 1);
+        out_mid_r = fftw_alloc_complex(audio.FFTmidbufferSize / 2 + 1);
+        memset(out_mid_l, 0, (audio.FFTmidbufferSize / 2 + 1) * sizeof(fftw_complex));
+        memset(out_mid_r, 0, (audio.FFTmidbufferSize / 2 + 1) * sizeof(fftw_complex));
 
         p_mid_l =
             fftw_plan_dft_r2c_1d(audio.FFTmidbufferSize, audio.in_mid_l, out_mid_l, FFTW_MEASURE);
@@ -424,15 +397,15 @@ as of 0.4.0 all options are specified in
 
         // TRIEBLE
         // audio.FFTtreblebufferSize =  audio.rate / treble_cut_off; // audio.FFTbassbufferSize;
-        audio.in_treble_r = fftw_alloc_real(2 * (audio.FFTtreblebufferSize / 2 + 1));
-        audio.in_treble_l = fftw_alloc_real(2 * (audio.FFTtreblebufferSize / 2 + 1));
-        memset(audio.in_treble_r, 0, 2 * (audio.FFTtreblebufferSize / 2 + 1) * sizeof(double));
-        memset(audio.in_treble_l, 0, 2 * (audio.FFTtreblebufferSize / 2 + 1) * sizeof(double));
-
-        out_treble_l = fftw_alloc_complex(2 * (audio.FFTtreblebufferSize / 2 + 1));
-        out_treble_r = fftw_alloc_complex(2 * (audio.FFTtreblebufferSize / 2 + 1));
-        memset(out_treble_l, 0, 2 * (audio.FFTtreblebufferSize / 2 + 1) * sizeof(fftw_complex));
-        memset(out_treble_r, 0, 2 * (audio.FFTtreblebufferSize / 2 + 1) * sizeof(fftw_complex));
+        audio.in_treble_r = fftw_alloc_real(audio.FFTtreblebufferSize);
+        audio.in_treble_l = fftw_alloc_real(audio.FFTtreblebufferSize);
+        audio.in_treble_r_raw = fftw_alloc_real(audio.FFTtreblebufferSize);
+        audio.in_treble_l_raw = fftw_alloc_real(audio.FFTtreblebufferSize);
+
+        out_treble_l = fftw_alloc_complex(audio.FFTtreblebufferSize / 2 + 1);
+        out_treble_r = fftw_alloc_complex(audio.FFTtreblebufferSize / 2 + 1);
+        memset(out_treble_l, 0, (audio.FFTtreblebufferSize / 2 + 1) * sizeof(fftw_complex));
+        memset(out_treble_r, 0, (audio.FFTtreblebufferSize / 2 + 1) * sizeof(fftw_complex));
 
         p_treble_l = fftw_plan_dft_r2c_1d(audio.FFTtreblebufferSize, audio.in_treble_l,
                                           out_treble_l, FFTW_MEASURE);
@@ -491,9 +464,7 @@ as of 0.4.0 all options are specified in
         case INPUT_PULSE:
             if (strcmp(audio.source, "auto") == 0) {
                 getPulseDefaultSink((void *)&audio);
-                sourceIsAuto = 1;
-            } else
-                sourceIsAuto = 0;
+            }
             // starting pulsemusic listener
             thr_id = pthread_create(&p_thread, NULL, input_pulse, (void *)&audio);
             audio.rate = 44100;
@@ -507,6 +478,22 @@ as of 0.4.0 all options are specified in
 #endif
         case INPUT_SHMEM:
             thr_id = pthread_create(&p_thread, NULL, input_shmem, (void *)&audio);
+
+            n = 0;
+
+            while (audio.rate == 0) {
+                req.tv_sec = 0;
+                req.tv_nsec = 1000000;
+                nanosleep(&req, NULL);
+                n++;
+                if (n > 2000) {
+                    cleanup();
+                    fprintf(stderr, "could not get rate and/or format, problems with audio thread? "
+                                    "quiting...\n");
+                    exit(EXIT_FAILURE);
+                }
+            }
+            debug("got format: %d and rate %d\n", audio.format, audio.rate);
             // audio.rate = 44100;
             break;
 #ifdef PORTAUDIO
@@ -521,13 +508,13 @@ as of 0.4.0 all options are specified in
 
         if (p.upper_cut_off > audio.rate / 2) {
             cleanup();
-            fprintf(stderr, "higher cuttoff frequency can't be higher then sample rate / 2");
+            fprintf(stderr, "higher cuttoff frequency can't be higher than sample rate / 2");
             exit(EXIT_FAILURE);
         }
 
         bool reloadConf = false;
 
-        while (!reloadConf) { // jumbing back to this loop means that you resized the screen
+        while (!reloadConf) { // jumping back to this loop means that you resized the screen
             for (n = 0; n < 256; n++) {
                 bars_last[n] = 0;
                 previous_frame[n] = 0;
@@ -537,20 +524,30 @@ as of 0.4.0 all options are specified in
                 bars[n] = 0;
             }
 
+            // frequencies on x axis require a bar width of four or more
+            if (p.xaxis == FREQUENCY && p.bar_width < 4)
+                p.bar_width = 4;
+
             switch (output_mode) {
 #ifdef NCURSES
             // output: start ncurses mode
             case OUTPUT_NCURSES:
                 init_terminal_ncurses(p.color, p.bcolor, p.col, p.bgcol, p.gradient,
                                       p.gradient_count, p.gradient_colors, &width, &lines);
+                if (p.xaxis != NONE)
+                    lines--;
                 // we have 8 times as much height due to using 1/8 block characters
                 height = lines * 8;
                 break;
 #endif
             case OUTPUT_NONCURSES:
                 get_terminal_dim_noncurses(&width, &lines);
+
+                if (p.xaxis != NONE)
+                    lines--;
+
                 init_terminal_noncurses(inAtty, p.col, p.bgcol, width, lines, p.bar_width);
-                height = (lines - 1) * 8;
+                height = lines * 8;
                 break;
 
             case OUTPUT_RAW:
@@ -603,7 +600,7 @@ as of 0.4.0 all options are specified in
                     p.autobars = 1;
             }
 
-            // getting orignial numbers of barss incase of resize
+            // getting original numbers of bars incase of resize
             if (p.autobars == 1) {
                 number_of_bars = (width + p.bar_spacing) / (p.bar_width + p.bar_spacing);
                 // if (p.bar_spacing != 0) number_of_bars = (width - number_of_bars * p.bar_spacing
@@ -643,7 +640,7 @@ as of 0.4.0 all options are specified in
 
             if (p.stereo)
                 number_of_bars =
-                    number_of_bars / 2; // in stereo onle half number of number_of_bars per channel
+                    number_of_bars / 2; // in stereo only half number of number_of_bars per channel
 
             if (p.userEQ_enabled && (number_of_bars > 0)) {
                 userEQ_keys_to_bars_ratio =
@@ -657,21 +654,35 @@ as of 0.4.0 all options are specified in
             // process: calculate cutoff frequencies and eq
             int bass_cut_off_bar = -1;
             int treble_cut_off_bar = -1;
-            bool first_bar = false;
+            bool first_bar = true;
             int first_treble_bar = 0;
+            int bar_buffer[number_of_bars + 1];
 
             for (n = 0; n < number_of_bars + 1; n++) {
                 double bar_distribution_coefficient = frequency_constant * (-1);
                 bar_distribution_coefficient +=
                     ((float)n + 1) / ((float)number_of_bars + 1) * frequency_constant;
                 cut_off_frequency[n] = p.upper_cut_off * pow(10, bar_distribution_coefficient);
+
+                if (n > 0) {
+                    if (cut_off_frequency[n - 1] >= cut_off_frequency[n] &&
+                        cut_off_frequency[n - 1] > bass_cut_off)
+                        cut_off_frequency[n] =
+                            cut_off_frequency[n - 1] +
+                            (cut_off_frequency[n - 1] - cut_off_frequency[n - 2]);
+                }
+
                 relative_cut_off[n] = cut_off_frequency[n] / (audio.rate / 2);
-                // remember nyquist!, pr my calculations this should be rate/2
-                // and  nyquist freq in M/2 but testing shows it is not...
+                // remember nyquist!, per my calculations this should be rate/2
+                // and nyquist freq in M/2 but testing shows it is not...
                 // or maybe the nq freq is in M/4
 
                 eq[n] = pow(cut_off_frequency[n], 1);
+
+                // the numbers that come out of the FFT are verry high
+                // the EQ is used to "normalize" them by dividing with this verry huge number
                 eq[n] *= (float)height / pow(2, 28);
+
                 if (p.userEQ_enabled)
                     eq[n] *= p.userEQ[(int)floor(((double)n) * userEQ_keys_to_bars_ratio)];
 
@@ -679,24 +690,25 @@ as of 0.4.0 all options are specified in
 
                 if (cut_off_frequency[n] < bass_cut_off) {
                     // BASS
+                    bar_buffer[n] = 1;
                     FFTbuffer_lower_cut_off[n] =
-                        relative_cut_off[n] * (audio.FFTbassbufferSize / 2) + 1;
+                        relative_cut_off[n] * (audio.FFTbassbufferSize / 2);
                     bass_cut_off_bar++;
                     treble_cut_off_bar++;
+                    if (bass_cut_off_bar > 0)
+                        first_bar = false;
 
                     eq[n] *= log2(audio.FFTbassbufferSize);
                 } else if (cut_off_frequency[n] > bass_cut_off &&
                            cut_off_frequency[n] < treble_cut_off) {
                     // MID
-                    FFTbuffer_lower_cut_off[n] =
-                        relative_cut_off[n] * (audio.FFTmidbufferSize / 2) + 1;
+                    bar_buffer[n] = 2;
+                    FFTbuffer_lower_cut_off[n] = relative_cut_off[n] * (audio.FFTmidbufferSize / 2);
                     treble_cut_off_bar++;
                     if ((treble_cut_off_bar - bass_cut_off_bar) == 1) {
                         first_bar = true;
                         FFTbuffer_upper_cut_off[n - 1] =
                             relative_cut_off[n] * (audio.FFTbassbufferSize / 2);
-                        if (FFTbuffer_upper_cut_off[n - 1] < FFTbuffer_lower_cut_off[n - 1])
-                            FFTbuffer_upper_cut_off[n - 1] = FFTbuffer_lower_cut_off[n - 1];
                     } else {
                         first_bar = false;
                     }
@@ -704,15 +716,14 @@ as of 0.4.0 all options are specified in
                     eq[n] *= log2(audio.FFTmidbufferSize);
                 } else {
                     // TREBLE
+                    bar_buffer[n] = 3;
                     FFTbuffer_lower_cut_off[n] =
-                        relative_cut_off[n] * (audio.FFTtreblebufferSize / 2) + 1;
+                        relative_cut_off[n] * (audio.FFTtreblebufferSize / 2);
                     first_treble_bar++;
                     if (first_treble_bar == 1) {
                         first_bar = true;
                         FFTbuffer_upper_cut_off[n - 1] =
                             relative_cut_off[n] * (audio.FFTmidbufferSize / 2);
-                        if (FFTbuffer_upper_cut_off[n - 1] < FFTbuffer_lower_cut_off[n - 1])
-                            FFTbuffer_upper_cut_off[n - 1] = FFTbuffer_lower_cut_off[n - 1];
                     } else {
                         first_bar = false;
                     }
@@ -720,14 +731,37 @@ as of 0.4.0 all options are specified in
                     eq[n] *= log2(audio.FFTtreblebufferSize);
                 }
 
-                if (n != 0 && !first_bar) {
-                    FFTbuffer_upper_cut_off[n - 1] = FFTbuffer_lower_cut_off[n] - 1;
+                if (n > 0) {
+                    if (!first_bar) {
+                        FFTbuffer_upper_cut_off[n - 1] = FFTbuffer_lower_cut_off[n] - 1;
+
+                        // pushing the spectrum up if the exponential function gets "clumped" in the
+                        // bass and caluclating new cut off frequencies
+                        if (FFTbuffer_lower_cut_off[n] <= FFTbuffer_lower_cut_off[n - 1]) {
+
+                            FFTbuffer_lower_cut_off[n] = FFTbuffer_lower_cut_off[n - 1] + 1;
+                            FFTbuffer_upper_cut_off[n - 1] = FFTbuffer_lower_cut_off[n] - 1;
+
+                            if (bar_buffer[n] == 1)
+                                relative_cut_off[n] = (float)(FFTbuffer_lower_cut_off[n]) /
+                                                      ((float)audio.FFTbassbufferSize / 2);
+                            else if (bar_buffer[n] == 2)
+                                relative_cut_off[n] = (float)(FFTbuffer_lower_cut_off[n]) /
+                                                      ((float)audio.FFTmidbufferSize / 2);
+                            else if (bar_buffer[n] == 3)
+                                relative_cut_off[n] = (float)(FFTbuffer_lower_cut_off[n]) /
+                                                      ((float)audio.FFTtreblebufferSize / 2);
 
-                    // pushing the spectrum up if the exponential function gets "clumped" in the
-                    // bass
-                    if (FFTbuffer_lower_cut_off[n] <= FFTbuffer_lower_cut_off[n - 1])
-                        FFTbuffer_lower_cut_off[n] = FFTbuffer_lower_cut_off[n - 1] + 1;
-                    FFTbuffer_upper_cut_off[n - 1] = FFTbuffer_lower_cut_off[n] - 1;
+                            cut_off_frequency[n] = relative_cut_off[n] * ((float)audio.rate / 2);
+                        }
+                    } else {
+                        if (FFTbuffer_upper_cut_off[n - 1] <= FFTbuffer_lower_cut_off[n - 1])
+                            FFTbuffer_upper_cut_off[n - 1] = FFTbuffer_lower_cut_off[n - 1] + 1;
+                    }
+                    upper_cut_off_frequency[n - 1] =
+                        cut_off_frequency[n]; // high_relative_cut_off * ((float)audio.rate / 2);
+                    center_frequencies[n - 1] =
+                        pow((cut_off_frequency[n - 1] * upper_cut_off_frequency[n - 1]), 0.5);
                 }
 
 #ifndef NDEBUG
@@ -746,8 +780,58 @@ as of 0.4.0 all options are specified in
             if (p.stereo)
                 number_of_bars = number_of_bars * 2;
 
+            int x_axis_info = 0;
+
+            if (p.xaxis != NONE) {
+                x_axis_info = 1;
+                double center_frequency;
+                if (output_mode == OUTPUT_NONCURSES) {
+                    printf("\r\033[%dB", lines + 1);
+                    if (rest)
+                        printf("\033[%dC", rest);
+                }
+                for (n = 0; n < number_of_bars; n++) {
+                    if (p.stereo) {
+                        if (n < number_of_bars / 2)
+                            center_frequency = center_frequencies[number_of_bars / 2 - 1 - n];
+                        else
+                            center_frequency = center_frequencies[n - number_of_bars / 2];
+                    } else {
+                        center_frequency = center_frequencies[n];
+                    }
+
+                    float freq_kilohz = center_frequency / 1000;
+                    int freq_floor = center_frequency;
+
+                    if (output_mode == OUTPUT_NCURSES) {
+#ifdef NCURSES
+                        if (center_frequency < 1000)
+                            mvprintw(lines, n * (p.bar_width + p.bar_spacing) + rest, "%-4d",
+                                     freq_floor);
+                        else if (center_frequency > 1000 && center_frequency < 10000)
+                            mvprintw(lines, n * (p.bar_width + p.bar_spacing) + rest, "%.2f",
+                                     freq_kilohz);
+                        else
+                            mvprintw(lines, n * (p.bar_width + p.bar_spacing) + rest, "%.1f",
+                                     freq_kilohz);
+#endif
+                    } else if (output_mode == OUTPUT_NONCURSES) {
+                        if (center_frequency < 1000)
+                            printf("%-4d", freq_floor);
+                        else if (center_frequency > 1000 && center_frequency < 10000)
+                            printf("%.2f", freq_kilohz);
+                        else
+                            printf("%.1f", freq_kilohz);
+
+                        if (n < number_of_bars - 1)
+                            printf("\033[%dC", p.bar_width + p.bar_spacing - 4);
+                    }
+                }
+                printf("\r\033[%dA", lines + 1);
+            }
+
             bool resizeTerminal = false;
-            fcntl(0, F_SETFL, O_NONBLOCK);
+            // fcntl(0, F_SETFL, O_NONBLOCK);
 
             if (p.framerate <= 1) {
                 req.tv_sec = 1 / (float)p.framerate;
@@ -763,8 +847,10 @@ as of 0.4.0 all options are specified in
                 if (output_mode == OUTPUT_NCURSES)
                     ch = getch();
 #endif
+                /*
                 if (output_mode == OUTPUT_NONCURSES)
                     ch = fgetc(stdin);
+                */
 
                 switch (ch) {
                 case 65: // key up
@@ -804,10 +890,8 @@ as of 0.4.0 all options are specified in
                     break;
 
                 case 'q':
-                    if (sourceIsAuto)
-                        free(audio.source);
-                    cleanup();
-                    return EXIT_SUCCESS;
+                    should_reload = 1;
+                    should_quit = 1;
                 }
 
                 if (should_reload) {
@@ -846,57 +930,87 @@ as of 0.4.0 all options are specified in
                     }
                 }
 
-                if (silence)
-                    sleep++;
-                else
-                    sleep = 0;
+                if (p.sleep_timer) {
+                    if (silence && sleep_counter <= p.framerate * p.sleep_timer)
+                        sleep_counter++;
+                    else if (!silence)
+                        sleep_counter = 0;
 
-                // process: if input was present for the last 5 seconds apply FFT to it
-                if (sleep < p.framerate * 5) {
+                    if (sleep_counter > p.framerate * p.sleep_timer) {
+#ifndef NDEBUG
+                        printw("no sound detected for 30 sec, going to sleep mode\n");
+#endif
+                        nanosleep(&sleep_mode_timer, NULL);
+                        continue;
+                    }
+                }
 
-                    // process: execute FFT and sort frequency bands
-                    if (p.stereo) {
-                        fftw_execute(p_bass_l);
-                        fftw_execute(p_bass_r);
-                        fftw_execute(p_mid_l);
-                        fftw_execute(p_mid_r);
-                        fftw_execute(p_treble_l);
-                        fftw_execute(p_treble_r);
-
-                        bars_left = separate_freq_bands(
-                            audio.FFTbassbufferSize, out_bass_l, audio.FFTmidbufferSize, out_mid_l,
-                            audio.FFTtreblebufferSize, out_treble_l, bass_cut_off_bar,
-                            treble_cut_off_bar, number_of_bars / 2, FFTbuffer_lower_cut_off,
-                            FFTbuffer_upper_cut_off, eq, LEFT_CHANNEL, p.sens, p.ignore);
-
-                        bars_right = separate_freq_bands(
-                            audio.FFTbassbufferSize, out_bass_r, audio.FFTmidbufferSize, out_mid_r,
-                            audio.FFTtreblebufferSize, out_treble_r, bass_cut_off_bar,
-                            treble_cut_off_bar, number_of_bars / 2, FFTbuffer_lower_cut_off,
-                            FFTbuffer_upper_cut_off, eq, RIGHT_CHANNEL, p.sens, p.ignore);
+                // process: execute FFT and sort frequency bands
+                pthread_mutex_lock(&lock);
+                fftw_execute(p_bass_l);
+                fftw_execute(p_mid_l);
+                fftw_execute(p_treble_l);
+                if (p.stereo) {
+                    fftw_execute(p_bass_r);
+                    fftw_execute(p_mid_r);
+                    fftw_execute(p_treble_r);
+                    number_of_bars /= 2;
+                }
+                pthread_mutex_unlock(&lock);
 
-                    } else {
-                        fftw_execute(p_bass_l);
-                        fftw_execute(p_mid_l);
-                        fftw_execute(p_treble_l);
-                        bars_mono = separate_freq_bands(
-                            audio.FFTbassbufferSize, out_bass_l, audio.FFTmidbufferSize, out_mid_l,
-                            audio.FFTtreblebufferSize, out_treble_l, bass_cut_off_bar,
-                            treble_cut_off_bar, number_of_bars, FFTbuffer_lower_cut_off,
-                            FFTbuffer_upper_cut_off, eq, LEFT_CHANNEL, p.sens, p.ignore);
+                // process: separate frequency bands
+                for (n = 0; n < number_of_bars; n++) {
+
+                    temp_l[n] = 0;
+                    if (p.stereo)
+                        temp_r[n] = 0;
+
+                    // process: add upp FFT values within bands
+                    for (int i = FFTbuffer_lower_cut_off[n]; i <= FFTbuffer_upper_cut_off[n]; i++) {
+                        if (n <= bass_cut_off_bar) {
+
+                            temp_l[n] += hypot(out_bass_l[i][0], out_bass_l[i][1]);
+
+                            if (p.stereo)
+                                temp_r[n] += hypot(out_bass_r[i][0], out_bass_r[i][1]);
+
+                        } else if (n > bass_cut_off_bar && n <= treble_cut_off_bar) {
+
+                            temp_l[n] += hypot(out_mid_l[i][0], out_mid_l[i][1]);
+
+                            if (p.stereo)
+                                temp_r[n] += hypot(out_mid_r[i][0], out_mid_r[i][1]);
+
+                        } else if (n > treble_cut_off_bar) {
+
+                            temp_l[n] += hypot(out_treble_l[i][0], out_treble_l[i][1]);
+
+                            if (p.stereo)
+                                temp_r[n] += hypot(out_treble_r[i][0], out_treble_r[i][1]);
+                        }
                     }
 
-                } else { //**if in sleep mode wait and continue**//
-#ifndef NDEBUG
-                    printw("no sound detected for 5 sec, going to sleep mode\n");
-#endif
-                    // wait 0.1 sec, then check sound again.
-                    sleep_mode_timer.tv_sec = 0;
-                    sleep_mode_timer.tv_nsec = 100000000;
-                    nanosleep(&sleep_mode_timer, NULL);
-                    continue;
-                }
+                    // getting average multiply with sens and eq
+                    temp_l[n] /= FFTbuffer_upper_cut_off[n] - FFTbuffer_lower_cut_off[n] + 1;
+                    temp_l[n] *= p.sens * eq[n];
+
+                    if (temp_l[n] <= p.ignore)
+                        temp_l[n] = 0;
+
+                    bars_left[n] = temp_l[n];
+
+                    if (p.stereo) {
+                        temp_r[n] /= FFTbuffer_upper_cut_off[n] - FFTbuffer_lower_cut_off[n] + 1;
+                        temp_r[n] *= p.sens * eq[n];
 
+                        if (temp_r[n] <= p.ignore)
+                            temp_r[n] = 0;
+
+                        bars_right[n] = temp_r[n];
+                    }
+                }
+                if (p.stereo)
+                    number_of_bars *= 2;
                 // process [filter]
 
                 if (p.monstercat) {
@@ -906,8 +1020,8 @@ as of 0.4.0 all options are specified in
                         bars_right = monstercat_filter(bars_right, number_of_bars / 2, p.waves,
                                                        p.monstercat);
                     } else {
-                        bars_mono =
-                            monstercat_filter(bars_mono, number_of_bars, p.waves, p.monstercat);
+                        bars_left =
+                            monstercat_filter(bars_left, number_of_bars, p.waves, p.monstercat);
                     }
                 }
 
@@ -925,7 +1039,7 @@ as of 0.4.0 all options are specified in
                         }
 
                     } else {
-                        bars[n] = bars_mono[n];
+                        bars[n] = bars_left[n];
                     }
 
                     // process [smoothing]: falloff
@@ -980,8 +1094,8 @@ as of 0.4.0 all options are specified in
                     if (output_mode != OUTPUT_RAW && bars[n] < 1)
                         bars[n] = 1;
 
-                    // autmatic sens adjustment
-                    if (p.autosens) {
+                    // automatic sense adjustment
+                    if (p.autosens && !silence) {
                         if (bars[n] > height && senselow) {
                             p.sens = p.sens * 0.98;
                             senselow = false;
@@ -996,6 +1110,8 @@ as of 0.4.0 all options are specified in
                 mvprintw(n + 1, 0, "sensitivity %.10e", p.sens);
                 mvprintw(n + 2, 0, "min value: %d\n", minvalue); // checking maxvalue 10000
                 mvprintw(n + 3, 0, "max value: %d\n", maxvalue); // checking maxvalue 10000
+                (void)rc;
+                (void)x_axis_info;
 #endif
 
 // output: draw processed input
@@ -1005,12 +1121,13 @@ as of 0.4.0 all options are specified in
 #ifdef NCURSES
                     rc = draw_terminal_ncurses(inAtty, lines, width, number_of_bars, p.bar_width,
                                                p.bar_spacing, rest, bars, previous_frame,
-                                               p.gradient);
+                                               p.gradient, x_axis_info);
                     break;
 #endif
                 case OUTPUT_NONCURSES:
                     rc = draw_terminal_noncurses(inAtty, lines, width, number_of_bars, p.bar_width,
-                                                 p.bar_spacing, rest, bars, previous_frame);
+                                                 p.bar_spacing, rest, bars, previous_frame,
+                                                 x_axis_info);
                     break;
                 case OUTPUT_RAW:
                     rc = print_raw_out(number_of_bars, fp, p.is_bin, p.bit_format, p.ascii_range,
@@ -1041,7 +1158,7 @@ as of 0.4.0 all options are specified in
 
         } // reloading config
         req.tv_sec = 0;
-        req.tv_nsec = 100; // waiting some time to make shure audio is ready
+        req.tv_nsec = 100; // waiting some time to make sure audio is ready
         nanosleep(&req, NULL);
 
         //**telling audio thread to terminate**//
@@ -1050,8 +1167,8 @@ as of 0.4.0 all options are specified in
 
         if (p.userEQ_enabled)
             free(p.userEQ);
-        if (sourceIsAuto)
-            free(audio.source);
+
+        free(audio.source);
 
         fftw_free(audio.in_bass_r);
         fftw_free(audio.in_bass_l);
@@ -1076,6 +1193,9 @@ as of 0.4.0 all options are specified in
 
         cleanup();
 
+        if (should_quit)
+            return EXIT_SUCCESS;
+
         // fclose(fp);
     }
 }
Binary files 0.7.2-1/cava.psf and 0.7.3-1/cava.psf differ
diff -pruN 0.7.2-1/config.c 0.7.3-1/config.c
--- 0.7.2-1/config.c	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/config.c	2020-10-07 16:30:24.000000000 +0000
@@ -22,7 +22,7 @@ enum input_method default_methods[] = {
     INPUT_PULSE,
 };
 
-char *outputMethod, *channels;
+char *outputMethod, *channels, *xaxisScale;
 
 const char *input_method_names[] = {
     "fifo", "portaudio", "alsa", "pulse", "sndio", "shmem",
@@ -52,18 +52,26 @@ void write_errorf(void *err, const char
     va_end(args);
 }
 
-int validate_color(char *checkColor, int om, void *err) {
+int validate_color(char *checkColor, void *params, void *err) {
+    struct config_params *p = (struct config_params *)params;
     struct error_s *error = (struct error_s *)err;
     int validColor = 0;
     if (checkColor[0] == '#' && strlen(checkColor) == 7) {
         // If the output mode is not ncurses, tell the user to use a named colour instead of hex
         // colours.
-        if (om != OUTPUT_NCURSES) {
-            write_errorf(
-                error, "Only 'ncurses' output method supports HTML colors. Please change "
-                       "the colours or the output method.\nAs of version 0.7.0 ncurses is no longer"
-                       " the default output method\n");
+        if (p->om != OUTPUT_NCURSES) {
+#ifdef NCURSES
+            write_errorf(error,
+                         "hex color configured, but ncurses not set. Forcing ncurses mode.\n");
+            p->om = OUTPUT_NCURSES;
+#else
+            write_errorf(error,
+                         "Only 'ncurses' output method supports HTML colors "
+                         "(required by gradient). "
+                         "Cava was built without ncurses support, install ncurses(w) dev files "
+                         "and rebuild.\n");
             return 0;
+#endif
         }
         // 0 to 9 and a to f
         for (int i = 1; checkColor[i]; ++i) {
@@ -94,14 +102,14 @@ bool validate_colors(void *params, void
     struct error_s *error = (struct error_s *)err;
 
     // validate: color
-    if (!validate_color(p->color, p->om, error)) {
+    if (!validate_color(p->color, p, error)) {
         write_errorf(error, "The value for 'foreground' is invalid. It can be either one of the 7 "
                             "named colors or a HTML color of the form '#xxxxxx'.\n");
         return false;
     }
 
     // validate: background color
-    if (!validate_color(p->bcolor, p->om, error)) {
+    if (!validate_color(p->bcolor, p, error)) {
         write_errorf(error, "The value for 'background' is invalid. It can be either one of the 7 "
                             "named colors or a HTML color of the form '#xxxxxx'.\n");
         return false;
@@ -109,7 +117,7 @@ bool validate_colors(void *params, void
 
     if (p->gradient) {
         for (int i = 0; i < p->gradient_count; i++) {
-            if (!validate_color(p->gradient_colors[i], p->om, error)) {
+            if (!validate_color(p->gradient_colors[i], p, error)) {
                 write_errorf(
                     error,
                     "Gradient color %d is invalid. It must be HTML color of the form '#xxxxxx'.\n",
@@ -226,6 +234,17 @@ bool validate_config(struct config_param
 #endif
     }
 
+    p->xaxis = NONE;
+    if (strcmp(xaxisScale, "none") == 0) {
+        p->xaxis = NONE;
+    }
+    if (strcmp(xaxisScale, "frequency") == 0) {
+        p->xaxis = FREQUENCY;
+    }
+    if (strcmp(xaxisScale, "note") == 0) {
+        p->xaxis = NOTE;
+    }
+
     // validate: output channels
     p->stereo = -1;
     if (strcmp(channels, "mono") == 0) {
@@ -267,6 +286,7 @@ bool validate_config(struct config_param
         return false;
     }
 
+    // validate: colors
     if (!validate_colors(p, error)) {
         return false;
     }
@@ -297,14 +317,6 @@ bool validate_config(struct config_param
     // setting sens
     p->sens = p->sens / 100;
 
-    // validate FFT buffer
-    if (p->FFTbufferSize >= 8 && p->FFTbufferSize <= 16) {
-        p->FFTbufferSize = pow(2, p->FFTbufferSize);
-    } else {
-        write_errorf(error, "FFT buffer is set in the exponent of 2 and must be between 8 - 16\n");
-        return false;
-    }
-
     return true;
 }
 
@@ -411,12 +423,13 @@ bool load_config(char configPath[PATH_MA
     }
 
 #ifdef NCURSES
-    outputMethod = (char *)iniparser_getstring(ini, "output:method", "noncurses");
+    outputMethod = (char *)iniparser_getstring(ini, "output:method", "ncurses");
 #endif
 #ifndef NCURSES
     outputMethod = (char *)iniparser_getstring(ini, "output:method", "noncurses");
 #endif
 
+    xaxisScale = (char *)iniparser_getstring(ini, "output:xaxis", "none");
     p->monstercat = 1.5 * iniparser_getdouble(ini, "smoothing:monstercat", 0);
     p->waves = iniparser_getint(ini, "smoothing:waves", 0);
     p->integral = iniparser_getdouble(ini, "smoothing:integral", 77);
@@ -436,7 +449,7 @@ bool load_config(char configPath[PATH_MA
     p->overshoot = iniparser_getint(ini, "general:overshoot", 20);
     p->lower_cut_off = iniparser_getint(ini, "general:lower_cutoff_freq", 50);
     p->upper_cut_off = iniparser_getint(ini, "general:higher_cutoff_freq", 10000);
-    p->FFTbufferSize = iniparser_getint(ini, "general:FFTbufferSize", 12);
+    p->sleep_timer = iniparser_getint(ini, "general:sleep_timer", 0);
 
     // config: output
     free(channels);
@@ -457,7 +470,7 @@ bool load_config(char configPath[PATH_MA
     p->userEQ_keys = iniparser_getsecnkeys(ini, "eq");
     if (p->userEQ_keys > 0) {
         p->userEQ_enabled = 1;
-        p->userEQ = calloc(p->userEQ_keys + 1, sizeof(p->userEQ));
+        p->userEQ = (double *)calloc(p->userEQ_keys + 1, sizeof(double));
 #ifndef LEGACYINIPARSER
         const char *keys[p->userEQ_keys];
         iniparser_getseckeys(ini, "eq", keys);
diff -pruN 0.7.2-1/config.h 0.7.3-1/config.h
--- 0.7.2-1/config.h	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/config.h	2020-10-07 16:30:24.000000000 +0000
@@ -45,6 +45,8 @@ enum input_method {
 
 enum output_method { OUTPUT_NCURSES, OUTPUT_NONCURSES, OUTPUT_RAW, OUTPUT_NOT_SUPORTED };
 
+enum xaxis_scale { NONE, FREQUENCY, NOTE };
+
 struct config_params {
     char *color, *bcolor, *raw_target, *audio_source,
         /**gradient_color_1, *gradient_color_2,*/ **gradient_colors, *data_format, *mono_option;
@@ -54,9 +56,10 @@ struct config_params {
     double *userEQ;
     enum input_method im;
     enum output_method om;
+    enum xaxis_scale xaxis;
     int userEQ_keys, userEQ_enabled, col, bgcol, autobars, stereo, is_bin, ascii_range, bit_format,
         gradient, gradient_count, fixedbars, framerate, bar_width, bar_spacing, autosens, overshoot,
-        waves, FFTbufferSize, fifoSample, fifoSampleBits;
+        waves, fifoSample, fifoSampleBits, sleep_timer;
 };
 
 struct error_s {
diff -pruN 0.7.2-1/debian/changelog 0.7.3-1/debian/changelog
--- 0.7.2-1/debian/changelog	2020-07-08 23:02:20.000000000 +0000
+++ 0.7.3-1/debian/changelog	2020-10-10 22:48:06.000000000 +0000
@@ -1,3 +1,9 @@
+cava-alsa (0.7.3-1) unstable; urgency=medium
+
+  * Updated to new upstream version.  
+
+ -- Lasse Flygenring-Harrsen <Lasse@Flygenring-Harrsen.com>  Sun, 11 Oct 2020 00:48:06 +0200
+
 cava-alsa (0.7.2-1) unstable; urgency=medium
 
   * Initial release. (Closes: #954325)
diff -pruN 0.7.2-1/debian/control 0.7.3-1/debian/control
--- 0.7.2-1/debian/control	2020-07-08 23:02:20.000000000 +0000
+++ 0.7.3-1/debian/control	2020-10-10 22:48:06.000000000 +0000
@@ -4,7 +4,7 @@ Priority: optional
 Maintainer: Lasse Flygenring-Harrsen <Lasse@Flygenring-Harrsen.com>
 Build-Depends: debhelper-compat (= 13), libfftw3-dev, libasound2-dev, libncursesw5-dev, libpulse-dev, libncurses5-dev, libtool-bin, libiniparser-dev
 Standards-Version: 4.5.0
-Homepage: https://karlstav.github.io/cava/
+Homepage: https://github.com/karlstav/cava
 Vcs-Browser: https://git.flygenring-harrsen.com/lasse/cava-alsa
 Vcs-Git: https://git.flygenring-harrsen.com/lasse/cava-alsa.git
 Rules-Requires-Root: no
diff -pruN 0.7.2-1/debian/patches/spelling-mistakes.diff 0.7.3-1/debian/patches/spelling-mistakes.diff
--- 0.7.2-1/debian/patches/spelling-mistakes.diff	2020-07-08 23:02:20.000000000 +0000
+++ 0.7.3-1/debian/patches/spelling-mistakes.diff	2020-10-10 22:48:06.000000000 +0000
@@ -8,10 +8,10 @@ Fixed spelling mistake 'quiting' instead
  1 file changed, 1 insertion(+), 1 deletion(-)
 
 diff --git a/cava.c b/cava.c
-index 62859bf..17a34a8 100644
+index 0a359bf..34d8d48 100644
 --- a/cava.c
 +++ b/cava.c
-@@ -474,7 +474,7 @@ as of 0.4.0 all options are specified in config file, see in '/home/username/.co
+@@ -489,7 +489,7 @@ as of 0.4.0 all options are specified in config file, see in '/home/username/.co
                  if (n > 2000) {
                      cleanup();
                      fprintf(stderr, "could not get rate and/or format, problems with audio thread? "
diff -pruN 0.7.2-1/debian/source/include-binaries 0.7.3-1/debian/source/include-binaries
--- 0.7.2-1/debian/source/include-binaries	1970-01-01 00:00:00.000000000 +0000
+++ 0.7.3-1/debian/source/include-binaries	2020-10-10 22:48:06.000000000 +0000
@@ -0,0 +1,2 @@
+example_files/cava.gif
+cava.psf
Binary files 0.7.2-1/example_files/cava.gif and 0.7.3-1/example_files/cava.gif differ
diff -pruN 0.7.2-1/example_files/config 0.7.3-1/example_files/config
--- 0.7.2-1/example_files/config	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/example_files/config	2020-10-07 16:30:24.000000000 +0000
@@ -33,13 +33,10 @@
 ; lower_cutoff_freq = 50
 ; higher_cutoff_freq = 10000
 
-# FFT buffer is set in the exponent of 2 and must be between 8 - 16, 8 = 256, 16 = 65536
-# this is the audio buffer used to create the spectrum
-# increasing the will improve the accuracy of the visualization,
-# but will also make it slower and increase CPU usage
-# anything below 10 and above 13 is considered experimental.
-; FFTbufferSize = 12;
 
+# Seconds with no input before cava goes to sleep mode. Cava will not perform FFT or drawing and
+# only check for input once per second. Cava will wake up once input is detected. 0 = disable.
+; sleep_timer = 0
 
 
 [input]
@@ -78,7 +75,7 @@
 
 # Output method. Can be 'ncurses', 'noncurses' or 'raw'.
 # 'noncurses' uses a custom framebuffer technique and draws only changes
-# from frame to frame. As of version 0.7.0 'noncurses' is default.
+# from frame to frame. 'ncurses' is default if supported
 #
 # 'raw' is an 8 or 16 bit (configurable via the 'bit_format' option) data
 # stream of the bar heights that can be used to send to other applications.
@@ -116,6 +113,7 @@
 # Colors can be one of seven predefined: black, blue, cyan, green, magenta, red, white, yellow.
 # Or defined by hex code '#xxxxxx' (hex code must be within ''). User defined colors requires
 # ncurses output method and a terminal that can change color definitions such as Gnome-terminal or rxvt.
+# if supported, ncurses mode will be forced on if user defined colors are used.
 # default is to keep current terminal color
 ; background = default
 ; foreground = default
diff -pruN 0.7.2-1/.github/ISSUE_TEMPLATE/bug_report.md 0.7.3-1/.github/ISSUE_TEMPLATE/bug_report.md
--- 0.7.2-1/.github/ISSUE_TEMPLATE/bug_report.md	1970-01-01 00:00:00.000000000 +0000
+++ 0.7.3-1/.github/ISSUE_TEMPLATE/bug_report.md	2020-10-07 16:30:24.000000000 +0000
@@ -0,0 +1,41 @@
+---
+name: Bug report
+about: Create a report to help us improve
+title: ''
+labels: ''
+assignees: ''
+
+---
+
+**When/where was the bug introduced?**
+- Are you using cava from a package repository, like AUR?
+- If so, check out the master branch here, is your issue already resolved?
+- Don't know how to clone, compile source code? File report to package maintainer instead.
+- try using git bisect to find out where the bug was introduced.
+
+**Describe the bug**
+A clear and concise description of what the bug is.
+
+**To Reproduce**
+Steps to reproduce the behavior:
+1. Set config parameter to
+2. run
+3. do something
+4. See error
+
+**Expected behavior**
+A clear and concise description of what you expected to happen.
+
+**Screenshots**
+If applicable, add screenshots to help explain your problem.
+
+**Desktop (please complete the following information):**
+ - OS: [e.g. macOS, Ubuntu]
+ - Version [e.g. 22]
+
+**Terminal emulator**
+- name
+- version
+
+**Additional context**
+Add any other context about the problem here.
diff -pruN 0.7.2-1/.github/ISSUE_TEMPLATE/feature_request.md 0.7.3-1/.github/ISSUE_TEMPLATE/feature_request.md
--- 0.7.2-1/.github/ISSUE_TEMPLATE/feature_request.md	1970-01-01 00:00:00.000000000 +0000
+++ 0.7.3-1/.github/ISSUE_TEMPLATE/feature_request.md	2020-10-07 16:30:24.000000000 +0000
@@ -0,0 +1,20 @@
+---
+name: Feature request
+about: Suggest an idea for this project
+title: ''
+labels: ''
+assignees: ''
+
+---
+
+**Is your feature request related to a problem? Please describe.**
+A clear and concise description of what the problem is. Ex. I'm always frustrated when [...]
+
+**Describe the solution you'd like**
+A clear and concise description of what you want to happen.
+
+**Describe alternatives you've considered**
+A clear and concise description of any alternative solutions or features you've considered.
+
+**Additional context**
+Add any other context or screenshots about the feature request here.
diff -pruN 0.7.2-1/input/alsa.c 0.7.3-1/input/alsa.c
--- 0.7.2-1/input/alsa.c	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/input/alsa.c	2020-10-07 16:30:24.000000000 +0000
@@ -97,7 +97,6 @@ const int size) {
         }
 }
 */
-#define FRAMES_NUMBER 256
 
 void *input_alsa(void *data) {
     int err;
@@ -105,7 +104,7 @@ void *input_alsa(void *data) {
     snd_pcm_t *handle;
     snd_pcm_uframes_t buffer_size;
     snd_pcm_uframes_t period_size;
-    snd_pcm_uframes_t frames = FRAMES_NUMBER;
+    snd_pcm_uframes_t frames = audio->FFTtreblebufferSize;
 
     initialize_audio_parameters(&handle, audio, &frames);
     snd_pcm_get_params(handle, &buffer_size, &period_size);
@@ -156,7 +155,9 @@ void *input_alsa(void *data) {
             debug("short read, read %d %d frames\n", err, (int)frames);
         }
 
-        write_to_fftw_input_buffers(buf, frames, data);
+        pthread_mutex_lock(&lock);
+        write_to_fftw_input_buffers(frames, buf, data);
+        pthread_mutex_unlock(&lock);
     }
 
     free(buffer);
diff -pruN 0.7.2-1/input/common.c 0.7.3-1/input/common.c
--- 0.7.2-1/input/common.c	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/input/common.c	2020-10-07 16:30:24.000000000 +0000
@@ -1,53 +1,91 @@
 #include "input/common.h"
+#include <math.h>
 
 #include <string.h>
 
 void reset_output_buffers(struct audio_data *data) {
-    memset(data->in_bass_r, 0, sizeof(double) * 2 * (data->FFTbassbufferSize / 2 + 1));
-    memset(data->in_bass_l, 0, sizeof(double) * 2 * (data->FFTbassbufferSize / 2 + 1));
-    memset(data->in_mid_r, 0, sizeof(double) * 2 * (data->FFTmidbufferSize / 2 + 1));
-    memset(data->in_mid_l, 0, sizeof(double) * 2 * (data->FFTmidbufferSize / 2 + 1));
-    memset(data->in_treble_r, 0, sizeof(double) * 2 * (data->FFTtreblebufferSize / 2 + 1));
-    memset(data->in_treble_l, 0, sizeof(double) * 2 * (data->FFTtreblebufferSize / 2 + 1));
+    memset(data->in_bass_r, 0, sizeof(double) * data->FFTbassbufferSize);
+    memset(data->in_bass_l, 0, sizeof(double) * data->FFTbassbufferSize);
+    memset(data->in_mid_r, 0, sizeof(double) * data->FFTmidbufferSize);
+    memset(data->in_mid_l, 0, sizeof(double) * data->FFTmidbufferSize);
+    memset(data->in_treble_r, 0, sizeof(double) * data->FFTtreblebufferSize);
+    memset(data->in_treble_l, 0, sizeof(double) * data->FFTtreblebufferSize);
+    memset(data->in_bass_r_raw, 0, sizeof(double) * data->FFTbassbufferSize);
+    memset(data->in_bass_l_raw, 0, sizeof(double) * data->FFTbassbufferSize);
+    memset(data->in_mid_r_raw, 0, sizeof(double) * data->FFTmidbufferSize);
+    memset(data->in_mid_l_raw, 0, sizeof(double) * data->FFTmidbufferSize);
+    memset(data->in_treble_r_raw, 0, sizeof(double) * data->FFTtreblebufferSize);
+    memset(data->in_treble_l_raw, 0, sizeof(double) * data->FFTtreblebufferSize);
 }
 
-int write_to_fftw_input_buffers(int16_t buf[], int16_t frames, void *data) {
+int write_to_fftw_input_buffers(int16_t frames, int16_t buf[frames * 2], void *data) {
+    if (frames == 0)
+        return 0;
     struct audio_data *audio = (struct audio_data *)data;
 
-    for (uint16_t i = 0; i < frames * 2; i += 2) {
+    for (uint16_t n = audio->FFTbassbufferSize; n > frames; n = n - frames) {
+        for (uint16_t i = 1; i <= frames; i++) {
+            audio->in_bass_l_raw[n - i] = audio->in_bass_l_raw[n - i - frames];
+            if (audio->channels == 2)
+                audio->in_bass_r_raw[n - i] = audio->in_bass_r_raw[n - i - frames];
+        }
+    }
+    for (uint16_t n = audio->FFTmidbufferSize; n > frames; n = n - frames) {
+        for (uint16_t i = 1; i <= frames; i++) {
+            audio->in_mid_l_raw[n - i] = audio->in_mid_l_raw[n - i - frames];
+            if (audio->channels == 2)
+                audio->in_mid_r_raw[n - i] = audio->in_mid_r_raw[n - i - frames];
+        }
+    }
+    for (uint16_t n = audio->FFTtreblebufferSize; n > frames; n = n - frames) {
+        for (uint16_t i = 1; i <= frames; i++) {
+            audio->in_treble_l_raw[n - i] = audio->in_treble_l_raw[n - i - frames];
+            if (audio->channels == 2)
+                audio->in_treble_r_raw[n - i] = audio->in_treble_r_raw[n - i - frames];
+        }
+    }
+    uint16_t n = frames - 1;
+    for (uint16_t i = 0; i < frames; i++) {
         if (audio->channels == 1) {
             if (audio->average) {
-                audio->in_bass_l[audio->bass_index] = (buf[i] + buf[i + 1]) / 2;
+                audio->in_bass_l_raw[n] = (buf[i * 2] + buf[i * 2 + 1]) / 2;
             }
             if (audio->left) {
-                audio->in_bass_l[audio->bass_index] = buf[i];
+                audio->in_bass_l_raw[n] = buf[i * 2];
             }
             if (audio->right) {
-                audio->in_bass_l[audio->bass_index] = buf[i + 1];
+                audio->in_bass_l_raw[n] = buf[i * 2 + 1];
             }
         }
         // stereo storing channels in buffer
         if (audio->channels == 2) {
-            audio->in_bass_l[audio->bass_index] = buf[i];
-            audio->in_bass_r[audio->bass_index] = buf[i + 1];
+            audio->in_bass_l_raw[n] = buf[i * 2];
+            audio->in_bass_r_raw[n] = buf[i * 2 + 1];
 
-            audio->in_mid_r[audio->mid_index] = audio->in_bass_r[audio->bass_index];
-            audio->in_treble_r[audio->treble_index] = audio->in_bass_r[audio->bass_index];
+            audio->in_mid_r_raw[n] = audio->in_bass_r_raw[n];
+            audio->in_treble_r_raw[n] = audio->in_bass_r_raw[n];
         }
 
-        audio->in_mid_l[audio->mid_index] = audio->in_bass_l[audio->bass_index];
-        audio->in_treble_l[audio->treble_index] = audio->in_bass_l[audio->bass_index];
-
-        audio->bass_index++;
-        audio->mid_index++;
-        audio->treble_index++;
-        if (audio->bass_index == audio->FFTbassbufferSize - 1)
-            audio->bass_index = 0;
-        if (audio->mid_index == audio->FFTmidbufferSize - 1)
-            audio->mid_index = 0;
-        if (audio->treble_index == audio->FFTtreblebufferSize - 1)
-            audio->treble_index = 0;
+        audio->in_mid_l_raw[n] = audio->in_bass_l_raw[n];
+        audio->in_treble_l_raw[n] = audio->in_bass_l_raw[n];
+        n--;
     }
 
+    // Hann Window
+    for (int i = 0; i < audio->FFTbassbufferSize; i++) {
+        audio->in_bass_l[i] = audio->bass_multiplier[i] * audio->in_bass_l_raw[i];
+        if (audio->channels == 2)
+            audio->in_bass_r[i] = audio->bass_multiplier[i] * audio->in_bass_r_raw[i];
+    }
+    for (int i = 0; i < audio->FFTmidbufferSize; i++) {
+        audio->in_mid_l[i] = audio->mid_multiplier[i] * audio->in_mid_l_raw[i];
+        if (audio->channels == 2)
+            audio->in_mid_r[i] = audio->mid_multiplier[i] * audio->in_mid_r_raw[i];
+    }
+    for (int i = 0; i < audio->FFTtreblebufferSize; i++) {
+        audio->in_treble_l[i] = audio->treble_multiplier[i] * audio->in_treble_l_raw[i];
+        if (audio->channels == 2)
+            audio->in_treble_r[i] = audio->treble_multiplier[i] * audio->in_treble_r_raw[i];
+    }
     return 0;
 }
diff -pruN 0.7.2-1/input/common.h 0.7.3-1/input/common.h
--- 0.7.2-1/input/common.h	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/input/common.h	2020-10-07 16:30:24.000000000 +0000
@@ -18,6 +18,12 @@ struct audio_data {
     int bass_index;
     int mid_index;
     int treble_index;
+    double *bass_multiplier;
+    double *mid_multiplier;
+    double *treble_multiplier;
+    double *in_bass_r_raw, *in_bass_l_raw;
+    double *in_mid_r_raw, *in_mid_l_raw;
+    double *in_treble_r_raw, *in_treble_l_raw;
     double *in_bass_r, *in_bass_l;
     double *in_mid_r, *in_mid_l;
     double *in_treble_r, *in_treble_l;
@@ -33,4 +39,6 @@ struct audio_data {
 
 void reset_output_buffers(struct audio_data *data);
 
-int write_to_fftw_input_buffers(int16_t buf[], int16_t frames, void *data);
+int write_to_fftw_input_buffers(int16_t frames, int16_t buf[frames * 2], void *data);
+
+extern pthread_mutex_t lock;
diff -pruN 0.7.2-1/input/fifo.c 0.7.3-1/input/fifo.c
--- 0.7.2-1/input/fifo.c	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/input/fifo.c	2020-10-07 16:30:24.000000000 +0000
@@ -3,8 +3,6 @@
 
 #include <time.h>
 
-#define SAMPLES_PER_BUFFER 512
-
 int open_fifo(const char *path) {
     int fd = open(path, O_RDONLY);
     int flags = fcntl(fd, F_GETFL, 0);
@@ -15,6 +13,7 @@ int open_fifo(const char *path) {
 // input: FIFO
 void *input_fifo(void *data) {
     struct audio_data *audio = (struct audio_data *)data;
+    int SAMPLES_PER_BUFFER = audio->FFTtreblebufferSize * 2;
     int bytes_per_sample = audio->format / 8;
     __attribute__((aligned(sizeof(uint16_t)))) uint8_t buf[SAMPLES_PER_BUFFER * bytes_per_sample];
     uint16_t *samples =
@@ -67,7 +66,9 @@ void *input_fifo(void *data) {
 
         // We worked with unsigned ints up until now to save on sign extension, but the FFT wants
         // signed ints.
-        write_to_fftw_input_buffers((int16_t *)samples, SAMPLES_PER_BUFFER / 2, audio);
+        pthread_mutex_lock(&lock);
+        write_to_fftw_input_buffers(SAMPLES_PER_BUFFER / 2, (int16_t *)samples, audio);
+        pthread_mutex_unlock(&lock);
     }
 
     close(fd);
diff -pruN 0.7.2-1/input/portaudio.c 0.7.3-1/input/portaudio.c
--- 0.7.2-1/input/portaudio.c	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/input/portaudio.c	2020-10-07 16:30:24.000000000 +0000
@@ -2,7 +2,6 @@
 #include "input/common.h"
 
 #include <portaudio.h>
-#define PORTBUFSIZE 512
 
 #define SAMPLE_SILENCE -32767
 #define PA_SAMPLE_TYPE paInt16
@@ -15,7 +14,7 @@ typedef struct {
 } paTestData;
 
 static struct audio_data *audio;
-// static int n = 0;
+int16_t silence_buffer[8092] = {SAMPLE_SILENCE};
 
 static int recordCallback(const void *inputBuffer, void *outputBuffer,
                           unsigned long framesPerBuffer, const PaStreamCallbackTimeInfo *timeInfo,
@@ -23,10 +22,8 @@ static int recordCallback(const void *in
     paTestData *data = (paTestData *)userData;
     SAMPLE *rptr = (SAMPLE *)inputBuffer;
     long framesToCalc;
-    // long i;
     int finished;
     unsigned long framesLeft = data->maxFrameIndex - data->frameIndex;
-    int16_t silence_buffer[PORTBUFSIZE] = {SAMPLE_SILENCE};
     (void)outputBuffer; // Prevent unused variable warnings.
     (void)timeInfo;
     (void)statusFlags;
@@ -40,41 +37,24 @@ static int recordCallback(const void *in
         finished = paContinue;
     }
 
-    if (inputBuffer == NULL) {
-        write_to_fftw_input_buffers(silence_buffer, framesToCalc, audio);
-        /*
-                        for(i=0; i<framesToCalc; i++) {
-                                if(audio->channels == 1) audio->audio_out_l[n] = SAMPLE_SILENCE;
-                                if(audio->channels == 2) {
-                                        audio->audio_out_l[n] = SAMPLE_SILENCE;
-                                        audio->audio_out_r[n] = SAMPLE_SILENCE;
-                                }
-                                if(n == PORTBUFSIZE-1) n = 0;
-                        }
-        */
-    } else {
-        write_to_fftw_input_buffers(rptr, framesToCalc, audio);
-        /*
-                        for(i=0; i<framesToCalc; i++) {
-                                if(audio->channels == 1) {
-                                        audio->audio_out_l[n] = (rptr[0] + rptr[1]) / 2;
-                                        rptr += 2;
-                                }
-                                if(audio->channels == 2) {
-                                        audio->audio_out_l[n] = *rptr++;
-                                        audio->audio_out_r[n] = *rptr++;
-                                }
-                                n++;
-                                if(n == PORTBUFSIZE-1) n = 0;
-                        }
-        */
-    }
+    pthread_mutex_lock(&lock);
+
+    if (inputBuffer == NULL)
+        write_to_fftw_input_buffers(framesToCalc, silence_buffer, audio);
+    else
+        write_to_fftw_input_buffers(framesToCalc, rptr, audio);
+
+    pthread_mutex_unlock(&lock);
 
     data->frameIndex += framesToCalc;
     if (finished == paComplete) {
         data->frameIndex = 0;
         finished = paContinue;
     }
+
+    if (audio->terminate == 1)
+        finished = paComplete;
+
     return finished;
 }
 
@@ -145,13 +125,13 @@ void *input_portaudio(void *audiodata) {
     inputParameters.device = deviceNum;
 
     // set parameters
-    data.maxFrameIndex = PORTBUFSIZE;
-    data.recordedSamples = (SAMPLE *)malloc(2 * PORTBUFSIZE * sizeof(SAMPLE));
+    data.maxFrameIndex = audio->FFTtreblebufferSize * 1024;
+    data.recordedSamples = (SAMPLE *)malloc(2 * data.maxFrameIndex * sizeof(SAMPLE));
     if (data.recordedSamples == NULL) {
         fprintf(stderr, "Error: failure in memory allocation!\n");
         exit(EXIT_FAILURE);
     } else
-        memset(data.recordedSamples, 0x00, 2 * PORTBUFSIZE);
+        memset(data.recordedSamples, 0x00, 2 * data.maxFrameIndex);
 
     inputParameters.channelCount = 2;
     inputParameters.sampleFormat = PA_SAMPLE_TYPE;
@@ -160,8 +140,8 @@ void *input_portaudio(void *audiodata) {
     inputParameters.hostApiSpecificStreamInfo = NULL;
 
     // set it to work
-    err = Pa_OpenStream(&stream, &inputParameters, NULL, audio->rate, PORTBUFSIZE, paClipOff,
-                        recordCallback, &data);
+    err = Pa_OpenStream(&stream, &inputParameters, NULL, audio->rate, audio->FFTtreblebufferSize,
+                        paClipOff, recordCallback, &data);
     if (err != paNoError) {
         fprintf(stderr, "Error: failure in opening stream (%x)\n", err);
         exit(EXIT_FAILURE);
@@ -179,7 +159,7 @@ void *input_portaudio(void *audiodata) {
 
         //  record
         while ((err = Pa_IsStreamActive(stream)) == 1) {
-            Pa_Sleep(5);
+            Pa_Sleep(1000);
             if (audio->terminate == 1)
                 break;
         }
diff -pruN 0.7.2-1/input/pulse.c 0.7.3-1/input/pulse.c
--- 0.7.2-1/input/pulse.c	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/input/pulse.c	2020-10-07 16:30:24.000000000 +0000
@@ -5,8 +5,6 @@
 #include <pulse/pulseaudio.h>
 #include <pulse/simple.h>
 
-#define BUFFERSIZE 4096
-
 pa_mainloop *m_pulseaudio_mainloop;
 
 void cb(__attribute__((unused)) pa_context *pulseaudio_context, const pa_server_info *i,
@@ -14,6 +12,7 @@ void cb(__attribute__((unused)) pa_conte
 
     // getting default sink name
     struct audio_data *audio = (struct audio_data *)userdata;
+    free(audio->source);
     audio->source = malloc(sizeof(char) * 1024);
 
     strcpy(audio->source, i->default_sink_name);
@@ -99,17 +98,20 @@ void getPulseDefaultSink(void *data) {
 void *input_pulse(void *data) {
 
     struct audio_data *audio = (struct audio_data *)data;
-    int16_t buf[BUFFERSIZE / 2];
+    uint16_t frames = audio->FFTtreblebufferSize;
+    int channels = 2;
+    int16_t buf[frames * channels];
 
     /* The sample type to use */
     static const pa_sample_spec ss = {.format = PA_SAMPLE_S16LE, .rate = 44100, .channels = 2};
 
     audio->format = 16;
 
-    static const pa_buffer_attr pb = {.maxlength = (uint32_t)-1, // BUFSIZE * 2,
-                                      .fragsize = BUFFERSIZE};
+    const int frag_size = frames * channels * audio->format / 8 *
+                          2; // we double this because of cpu performance issues with pulseaudio
 
-    uint16_t frames = BUFFERSIZE / 4;
+    pa_buffer_attr pb = {.maxlength = (uint32_t)-1, // BUFSIZE * 2,
+                         .fragsize = frag_size};
 
     pa_simple *s = NULL;
     int error;
@@ -121,7 +123,6 @@ void *input_pulse(void *data) {
                 audio->source, pa_strerror(error));
 
         audio->terminate = 1;
-        pthread_exit(NULL);
     }
 
     while (!audio->terminate) {
@@ -129,14 +130,14 @@ void *input_pulse(void *data) {
             sprintf(audio->error_message, __FILE__ ": pa_simple_read() failed: %s\n",
                     pa_strerror(error));
             audio->terminate = 1;
-            pthread_exit(NULL);
         }
 
-        // sorting out channels
-
-        write_to_fftw_input_buffers(buf, frames, data);
+        pthread_mutex_lock(&lock);
+        write_to_fftw_input_buffers(frames, buf, data);
+        pthread_mutex_unlock(&lock);
     }
 
     pa_simple_free(s);
+    pthread_exit(NULL);
     return 0;
 }
diff -pruN 0.7.2-1/input/shmem.c 0.7.3-1/input/shmem.c
--- 0.7.2-1/input/shmem.c	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/input/shmem.c	2020-10-07 16:30:24.000000000 +0000
@@ -7,14 +7,13 @@
 
 typedef unsigned int u32_t;
 typedef short s16_t;
-
-// #define BUFSIZE 1024
-#define BUFSIZE 2048
-
 int rc;
 
+// See issue #375
+// Hard-coded in squeezelite's output_vis.c, but
+// this should be the same as mmap_area->buf_size
+// if you were to dynamically allocate.
 #define VIS_BUF_SIZE 16384
-#define VB_OFFSET 8192 + 4096
 
 typedef struct {
     pthread_rwlock_t rwlock;
@@ -32,6 +31,12 @@ void *input_shmem(void *data) {
     vis_t *mmap_area;
     int fd; /* file descriptor to mmaped area */
     int mmap_count = sizeof(vis_t);
+    int buf_frames;
+    struct timespec req = {.tv_sec = 0, .tv_nsec = 0};
+
+    s16_t silence_buffer[VIS_BUF_SIZE];
+    for (int i = 0; i < VIS_BUF_SIZE; i++)
+        silence_buffer[i] = 0;
 
     printf("input_shmem: source: %s", audio->source);
 
@@ -47,23 +52,27 @@ void *input_shmem(void *data) {
             exit(EXIT_FAILURE);
         }
     }
-    // printf("bufs: %u / run: %u / rate: %u\n",mmap_area->buf_size, mmap_area->running,
-    // mmap_area->rate);
-    audio->rate = mmap_area->rate;
 
     while (!audio->terminate) {
-        write_to_fftw_input_buffers(mmap_area->buffer, BUFSIZE, audio);
-        /*
-                        for (i = VB_OFFSET; i < BUFSIZE+VB_OFFSET; i += 2) {
-                                if (audio->channels == 1) {
-                                        audio->audio_out_l[n] = (mmap_area->buffer[i] +
-           mmap_area->buffer[i + 1]) / 2; } else if (audio->channels == 2) { audio->audio_out_l[n] =
-           mmap_area->buffer[i]; audio->audio_out_r[n] = mmap_area->buffer[i + 1];
-                                }
-                                n++;
-                                if (n == audio->FFTbufferSize - 1) n = 0;
-                        }
-        */
+        // audio rate may change between songs (e.g. 44.1kHz to 96kHz)
+        audio->rate = mmap_area->rate;
+        buf_frames = mmap_area->buf_size / 2;
+        // reread after the whole buffer has changed
+        req.tv_nsec = (1000000 / mmap_area->rate) * buf_frames;
+        if (mmap_area->running) {
+            // Frames are written by squeezelite to the buffer array starting from
+            // buffer[buf_index + 1], looping around the whole array.
+            // Thus, the starting position only affects the phase spectrum of the
+            // fft, and not the power spectrum, so we can just read in the
+            // whole buffer.
+            pthread_mutex_lock(&lock);
+            write_to_fftw_input_buffers(buf_frames, mmap_area->buffer, audio);
+            pthread_mutex_unlock(&lock);
+            nanosleep(&req, NULL);
+        } else {
+            write_to_fftw_input_buffers(buf_frames, silence_buffer, audio);
+            nanosleep(&req, NULL);
+        }
     }
 
     // cleanup
diff -pruN 0.7.2-1/input/sndio.c 0.7.3-1/input/sndio.c
--- 0.7.2-1/input/sndio.c	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/input/sndio.c	2020-10-07 16:30:24.000000000 +0000
@@ -7,8 +7,7 @@ void *input_sndio(void *data) {
     struct audio_data *audio = (struct audio_data *)data;
     struct sio_par par;
     struct sio_hdl *hdl;
-    int16_t buf[256];
-    unsigned int channels;
+    int16_t buf[audio->FFTtreblebufferSize * 2];
 
     sio_initpar(&par);
     par.sig = 1;
@@ -34,26 +33,16 @@ void *input_sndio(void *data) {
         exit(EXIT_FAILURE);
     }
 
-    uint16_t frames = (sizeof(buf) / sizeof(buf[0])) / channels;
+    uint16_t frames = (sizeof(buf) / sizeof(buf[0])) / 2;
     while (audio->terminate != 1) {
         if (sio_read(hdl, buf, sizeof(buf)) == 0) {
             fprintf(stderr, __FILE__ ": sio_read() failed: %s\n", strerror(errno));
             exit(EXIT_FAILURE);
         }
 
-        write_to_fftw_input_buffers(buf, frames, audio);
-        /*
-                        for (i = 0; i < sizeof(buf)/sizeof(buf[0]); i += 2) {
-                                if (par.rchan == 1) {
-                                        // sndiod has already taken care of averaging the samples
-                                        audio->audio_out_l[n] = buf[i];
-                                } else if (par.rchan == 2) {
-                                        audio->audio_out_l[n] = buf[i];
-                                        audio->audio_out_r[n] = buf[i + 1];
-                                }
-                                n = (n + 1) % audio->FFTbufferSize;
-                        }
-        */
+        pthread_mutex_lock(&lock);
+        write_to_fftw_input_buffers(frames, buf, audio);
+        pthread_mutex_unlock(&lock);
     }
 
     sio_stop(hdl);
diff -pruN 0.7.2-1/output/terminal_ncurses.c 0.7.3-1/output/terminal_ncurses.c
--- 0.7.2-1/output/terminal_ncurses.c	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/output/terminal_ncurses.c	2020-10-07 16:30:24.000000000 +0000
@@ -20,6 +20,10 @@ struct colors {
 
 #define MAX_COLOR_REDEFINITION 256
 
+const wchar_t *bar_heights[] = {L"\u2581", L"\u2582", L"\u2583", L"\u2584",
+                                L"\u2585", L"\u2586", L"\u2587", L"\u2588"};
+int num_bar_heights = (sizeof(bar_heights) / sizeof(bar_heights[0]));
+
 // static struct colors the_color_redefinitions[MAX_COLOR_REDEFINITION];
 
 static void parse_color(char *color_string, struct colors *color) {
@@ -188,24 +192,25 @@ void get_terminal_dim_ncurses(int *width
 #define TERMINAL_RESIZED -1
 
 int draw_terminal_ncurses(int is_tty, int terminal_height, int terminal_width, int bars_count,
-                          int bar_width, int bar_spacing, int rest, const int f[200],
-                          int flastd[200], int gradient) {
+                          int bar_width, int bar_spacing, int rest, const int bars[256],
+                          int previous_frame[200], int gradient, int x_axis_info) {
     const int height = terminal_height - 1;
-    const wchar_t *bar_heights[] = {L"\u2581", L"\u2582", L"\u2583", L"\u2584",
-                                    L"\u2585", L"\u2586", L"\u2587", L"\u2588"};
-    int num_bar_heights = (sizeof(bar_heights) / sizeof(bar_heights[0]));
 
     // output: check if terminal has been resized
     if (!is_tty) {
+        if (x_axis_info)
+            terminal_height++;
         if (LINES != terminal_height || COLS != terminal_width) {
             return TERMINAL_RESIZED;
+            if (x_axis_info)
+                terminal_height--;
         }
     }
 
     // Compute how much of the screen we possibly need to update ahead-of-time.
     int max_update_y = 0;
     for (int bar = 0; bar < bars_count; bar++) {
-        max_update_y = max(max_update_y, max(f[bar], flastd[bar]));
+        max_update_y = max(max_update_y, max(bars[bar], previous_frame[bar]));
     }
 
     max_update_y = (max_update_y + num_bar_heights) / num_bar_heights;
@@ -216,20 +221,20 @@ int draw_terminal_ncurses(int is_tty, in
         }
 
         for (int bar = 0; bar < bars_count; bar++) {
-            if (f[bar] == flastd[bar]) {
+            if (bars[bar] == previous_frame[bar]) {
                 continue;
             }
 
             int cur_col = bar * bar_width + bar * bar_spacing + rest;
-            int f_cell = (f[bar] - 1) / num_bar_heights;
-            int f_last_cell = (flastd[bar] - 1) / num_bar_heights;
+            int f_cell = (bars[bar] - 1) / num_bar_heights;
+            int f_last_cell = (previous_frame[bar] - 1) / num_bar_heights;
 
             if (f_cell >= y) {
                 int bar_step;
 
                 if (f_cell == y) {
                     // The "cap" of the bar occurs at this [y].
-                    bar_step = (f[bar] - 1) % num_bar_heights;
+                    bar_step = (bars[bar] - 1) % num_bar_heights;
                 } else if (f_last_cell <= y) {
                     // The bar is full at this [y].
                     bar_step = num_bar_heights - 1;
@@ -240,7 +245,7 @@ int draw_terminal_ncurses(int is_tty, in
 
                 for (int col = cur_col, i = 0; i < bar_width; i++, col++) {
                     if (is_tty) {
-                        mvaddch(height - y, col, '1' + bar_step);
+                        mvaddch(height - y, col, 0x41 + bar_step);
                     } else {
                         mvaddwstr(height - y, col, bar_heights[bar_step]);
                     }
diff -pruN 0.7.2-1/output/terminal_ncurses.h 0.7.3-1/output/terminal_ncurses.h
--- 0.7.2-1/output/terminal_ncurses.h	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/output/terminal_ncurses.h	2020-10-07 16:30:24.000000000 +0000
@@ -3,6 +3,6 @@ void init_terminal_ncurses(char *const f
                            int gradient_count, char **gradient_colors, int *width, int *height);
 void get_terminal_dim_ncurses(int *width, int *height);
 int draw_terminal_ncurses(int is_tty, int terminal_height, int terminal_width, int bars_count,
-                          int bar_width, int bar_spacing, int rest, const int f[200],
-                          int flastd[200], int gradient);
+                          int bar_width, int bar_spacing, int rest, const int bars[256],
+                          int previous_frame[256], int gradient, int x_axis_info);
 void cleanup_terminal_ncurses(void);
diff -pruN 0.7.2-1/output/terminal_noncurses.c 0.7.3-1/output/terminal_noncurses.c
--- 0.7.2-1/output/terminal_noncurses.c	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/output/terminal_noncurses.c	2020-10-07 16:30:24.000000000 +0000
@@ -35,17 +35,31 @@ int setecho(int fd, int onoff) {
     return 0;
 }
 
+// general: cleanup
+void free_terminal_noncurses(void) {
+    free(frame_buffer);
+    free(ttyframe_buffer);
+    free(spacestring);
+    free(ttyspacestring);
+    for (int i = 0; i < 8; i++) {
+        free(barstring[i]);
+        free(ttybarstring[i]);
+    }
+}
+
 int init_terminal_noncurses(int tty, int col, int bgcol, int width, int lines, int bar_width) {
 
+    free_terminal_noncurses();
+
     if (tty) {
 
         ttybuf_length = sizeof(char) * width * lines * 10;
         ttyframe_buffer = (char *)malloc(ttybuf_length);
-        ttyspacestring = (char *)malloc(sizeof(char) * bar_width);
+        ttyspacestring = (char *)malloc(sizeof(char) * (bar_width + 1));
 
         // clearing barstrings
         for (int n = 0; n < 8; n++) {
-            ttybarstring[n] = (char *)malloc(sizeof(char) * bar_width);
+            ttybarstring[n] = (char *)malloc(sizeof(char) * (bar_width + 1));
             ttybarstring[n][0] = '\0';
         }
         ttyspacestring[0] = '\0';
@@ -53,25 +67,25 @@ int init_terminal_noncurses(int tty, int
 
         // creating barstrings for drawing
         for (int n = 0; n < bar_width; n++) {
-            strcat(ttybarstring[0], "8");
-            strcat(ttybarstring[1], "1");
-            strcat(ttybarstring[2], "2");
-            strcat(ttybarstring[3], "3");
-            strcat(ttybarstring[4], "4");
-            strcat(ttybarstring[5], "5");
-            strcat(ttybarstring[6], "6");
-            strcat(ttybarstring[7], "7");
+            strcat(ttybarstring[0], "H");
+            strcat(ttybarstring[1], "A");
+            strcat(ttybarstring[2], "B");
+            strcat(ttybarstring[3], "C");
+            strcat(ttybarstring[4], "D");
+            strcat(ttybarstring[5], "E");
+            strcat(ttybarstring[6], "F");
+            strcat(ttybarstring[7], "G");
             strcat(ttyspacestring, " ");
         }
     } else if (!tty) {
 
         buf_length = sizeof(wchar_t) * width * lines * 10;
         frame_buffer = (wchar_t *)malloc(buf_length);
-        spacestring = (wchar_t *)malloc(sizeof(wchar_t) * bar_width);
+        spacestring = (wchar_t *)malloc(sizeof(wchar_t) * (bar_width + 1));
 
         // clearing barstrings
         for (int n = 0; n < 8; n++) {
-            barstring[n] = (wchar_t *)malloc(sizeof(wchar_t) * bar_width);
+            barstring[n] = (wchar_t *)malloc(sizeof(wchar_t) * (bar_width + 1));
             barstring[n][0] = '\0';
         }
         spacestring[0] = '\0';
@@ -110,7 +124,7 @@ int init_terminal_noncurses(int tty, int
         bgcol += 40;
         printf("\033[%dm", bgcol);
 
-        for (int n = (lines); n >= 0; n--) {
+        for (int n = lines; n >= 0; n--) {
             for (int i = 0; i < width; i++) {
 
                 printf(" "); // setting backround color
@@ -141,7 +155,8 @@ void get_terminal_dim_noncurses(int *wid
 }
 
 int draw_terminal_noncurses(int tty, int lines, int width, int number_of_bars, int bar_width,
-                            int bar_spacing, int rest, int bars[200], int previous_frame[200]) {
+                            int bar_spacing, int rest, int bars[256], int previous_frame[256],
+                            int x_axis_info) {
 
     int current_cell, prev_cell, same_line, new_line, cx;
 
@@ -150,20 +165,18 @@ int draw_terminal_noncurses(int tty, int
     same_line = 0;
     new_line = 0;
     cx = 0;
-    if (!tty) {
 
+    if (!tty) {
         // output: check if terminal has been resized
         ioctl(STDOUT_FILENO, TIOCGWINSZ, &dim);
-
-        if ((int)dim.ws_row != lines || (int)dim.ws_col != width) {
-            free(frame_buffer);
-            free(spacestring);
-            for (int i = 0; i < 8; i++)
-                free(barstring[i]);
-
+        if (x_axis_info)
+            lines++;
+        if ((int)dim.ws_row != (lines) || (int)dim.ws_col != width)
             return -1;
-        }
+        if (x_axis_info)
+            lines--;
     }
+
     if (tty)
         ttyframe_buffer[0] = '\0';
     else if (!tty)
@@ -198,7 +211,7 @@ int draw_terminal_noncurses(int tty, int
                         same_bar = 0;
                     }
 
-                    if (!center_adjusted) {
+                    if (!center_adjusted && rest) {
                         cx += snprintf(ttyframe_buffer + cx, ttybuf_length - cx, "\033[%dC", rest);
                         center_adjusted = 1;
                     }
@@ -213,8 +226,9 @@ int draw_terminal_noncurses(int tty, int
                         cx += snprintf(ttyframe_buffer + cx, ttybuf_length - cx, "%s",
                                        ttybarstring[current_cell]);
 
-                    cx +=
-                        snprintf(ttyframe_buffer + cx, ttybuf_length - cx, "\033[%dC", bar_spacing);
+                    if (bar_spacing)
+                        cx += snprintf(ttyframe_buffer + cx, ttybuf_length - cx, "\033[%dC",
+                                       bar_spacing);
                 } else if (!tty) {
                     if (same_line > 0) {
                         cx += swprintf(frame_buffer + cx, buf_length - cx, L"\033[%dB",
@@ -241,7 +255,9 @@ int draw_terminal_noncurses(int tty, int
                     else
                         cx += swprintf(frame_buffer + cx, buf_length - cx, barstring[current_cell]);
 
-                    cx += swprintf(frame_buffer + cx, buf_length - cx, L"\033[%dC", bar_spacing);
+                    if (bar_spacing)
+                        cx +=
+                            swprintf(frame_buffer + cx, buf_length - cx, L"\033[%dC", bar_spacing);
                 }
             }
         }
@@ -270,16 +286,7 @@ int draw_terminal_noncurses(int tty, int
     return 0;
 }
 
-// general: cleanup
 void cleanup_terminal_noncurses(void) {
-    free(frame_buffer);
-    free(ttyframe_buffer);
-    free(spacestring);
-    free(ttyspacestring);
-    for (int i = 0; i < 8; i++) {
-        free(barstring[i]);
-        free(ttybarstring[i]);
-    }
     setecho(STDIN_FILENO, 1);
     printf("\033[0m\n");
     system("setfont  >/dev/null 2>&1");
diff -pruN 0.7.2-1/output/terminal_noncurses.h 0.7.3-1/output/terminal_noncurses.h
--- 0.7.2-1/output/terminal_noncurses.h	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/output/terminal_noncurses.h	2020-10-07 16:30:24.000000000 +0000
@@ -1,5 +1,6 @@
-int init_terminal_noncurses(int tty, int col, int bgcol, int w, int h, int bar_width);
+int init_terminal_noncurses(int inAtty, int col, int bgcol, int w, int h, int bar_width);
 void get_terminal_dim_noncurses(int *w, int *h);
-int draw_terminal_noncurses(int virt, int height, int width, int bars, int bar_width, int bs,
-                            int rest, int f[200], int flastd[200]);
+int draw_terminal_noncurses(int inAtty, int lines, int width, int number_of_bars, int bar_width,
+                            int bar_spacing, int rest, int bars[256], int previous_frame[256],
+                            int x_axis_info);
 void cleanup_terminal_noncurses(void);
diff -pruN 0.7.2-1/README.md 0.7.3-1/README.md
--- 0.7.2-1/README.md	2020-07-01 18:23:22.000000000 +0000
+++ 0.7.3-1/README.md	2020-10-07 16:30:24.000000000 +0000
@@ -9,9 +9,7 @@ Now also works on macOS!
 
 by [Karl Stavestrand](mailto:karl@stavestrand.no)
 
-![spectrum](https://raw.githubusercontent.com/karlstav/cava/gh-pages/cava_gradient.gif "spectrum")
-
-thanks to [anko](https://github.com/anko) for the gif, here is the [recipe]( http://unix.stackexchange.com/questions/113695/gif-screencastng-the-unix-way).
+![spectrum](https://github.com/karlstav/cava/blob/master/example_files/cava.gif "spectrum")
 
 [Demo video](https://youtu.be/9PSp8VA6yjU)
 
@@ -31,6 +29,7 @@ thanks to [anko](https://github.com/anko
     - [Fedora](#fedora)
     - [Arch](#arch)
     - [Ubuntu](#ubuntu)
+    - [Debian (unstable)](#debian-unstable)
 - [Capturing audio](#capturing-audio)
   - [From Pulseaudio monitor source (Easy, default if supported)](#from-pulseaudio-monitor-source-easy-default-if-supported)
   - [From ALSA-loopback device (Tricky)](#from-alsa-loopback-device-tricky)
@@ -192,33 +191,33 @@ Cava is in [AUR](https://aur.archlinux.o
 
 #### Ubuntu
 
-Michael Nguyen has added CAVA to his PPA, it can be installed with:
-
-    sudo add-apt-repository ppa:tehtotalpwnage/ppa
-    sudo apt-get update
-    sudo apt-get install cava
-    
-For Ubuntu 18 or newer, you can use Harshal Sheth's PPA:
+Harshal Sheth has added CAVA to his PPA, it can be installed with:
 
-    sudo add-apt-repository ppa:hsheth2/ppa
-    sudo apt-get update
-    sudo apt-get install cava
+    add-apt-repository ppa:hsheth2/ppa
+    apt-get update
+    apt-get install cava
 
 All distro specific instalation sources might be out of date.
 
+#### Debian (unstable)
+
+A package has now been introduced into Debian Unstable.
+
+    apt-get install cava
+    
+It should move into Debian testing and stable over time, and will therefore also move into the repositories of Ubuntu and its derivatives. For more information see issue [#373](https://github.com/karlstav/cava/issues/373).
+
 
 Capturing audio
 ---------------
 
 ### From Pulseaudio monitor source (Easy, default if supported)
 
-First make sure you have installed pulseaudio dev files and that cava has been built with pulseaudio support (it should be automatically if the dev files are found).
-
-If you're lucky all you have to do is to uncomment this line in the config file under input:
+Just make sure you have installed pulseaudio dev files and that cava has been built with pulseaudio support (it should be automatically if the dev files are found).
 
-    method = pulse
+If you're lucky all you have to do is to run cava.
  
-If nothing happens you might have to use a different source than the default. The default might also be your microphone. Look at the config file for help. 
+If nothing happens you might have to use a different source than the default. The default might also be your microphone. Look at the [config](#configuration) file for help. 
 
 
 ### From ALSA-loopback device (Tricky)
@@ -227,7 +226,7 @@ Set
 
     method = alsa
 
-in the config file.
+in the [config](#configuration) file.
 
 ALSA can be difficult because there is no native way to grab audio from an output. If you want to capture audio straight fom the output (not just mic or line-in), you must create an ALSA loopback interface, then output the audio simultaneously to both the loopback and your normal interface.
 
@@ -262,7 +261,7 @@ Add these lines in mpd:
         format                  "44100:16:2"
     }
 
-Uncomment and change input method to `fifo` in the config file.
+Uncomment and change input method to `fifo` in the [config](#configuration) file.
 
 The path of the fifo can be specified with the `source` parameter.
 
@@ -291,7 +290,7 @@ $ AUDIODEVICE=snd/0.monitor cava
 
 ### squeezelite
 [squeezelite](https://en.wikipedia.org/wiki/Squeezelite) is one of several software clients available for the Logitech Media Server. Squeezelite can export its audio data as shared memory, which is what this input module uses.
-Just adapt your config:
+Just adapt your [config](#configuration):
 ```
 method = shmem
 source = /squeezelite-AA:BB:CC:DD:EE:FF
@@ -300,16 +299,28 @@ where `AA:BB:CC:DD:EE:FF` is squeezelite
 Note: squeezelite must be started with the `-v` flag to enable visualizer support.
 
 ### macOS
-Install [Soundflower](https://github.com/mattingalls/Soundflower) to create a loopback interface. Use Audio MIDI Setup to configure a virtual interface that outputs audio to both your speakers and the loopbacl interface, following [this](https://github.com/RogueAmoeba/Soundflower-Original/issues/44#issuecomment-151586106) recipe.
 
-Then edit your config to use this interface with portaudio:
+Note: cava looks no good in the default macOS terminal. For a better look install [kitty](https://sw.kovidgoyal.net/kitty/index.html). Be where that you might run into #109, that can be fixed like [this](https://stackoverflow.com/questions/7165108/in-os-x-lion-lang-is-not-set-to-utf-8-how-to-fix-it).
+
+**Background Music**
+
+Install [Background Music](https://github.com/kyleneideck/BackgroundMusic) which provides a loopback interface automatically. Once installed and running just edit your [config](#configuration) to use this interface with portaudio:
 
 ```
 method = portaudio
-source = "Soundflower (2ch)"
+source = "Background Music"
 ```
 
-Note: cava looks no good in the default macOS terminal. For a better look install [kitty](https://sw.kovidgoyal.net/kitty/index.html). Be where that you might run into #109, that can be fixed like [this](https://stackoverflow.com/questions/7165108/in-os-x-lion-lang-is-not-set-to-utf-8-how-to-fix-it).
+**Sound Flower**
+
+[Soundflower](https://github.com/mattingalls/Soundflower) also works to create a loopback interface. Use Audio MIDI Setup to configure a virtual interface that outputs audio to both your speakers and the loopback interface, following [this](https://github.com/RogueAmoeba/Soundflower-Original/issues/44#issuecomment-151586106) recipe. By creating a multi-output device you lose the ability to control the volume on your keyboard. Because of this, we recommend the Background Music app which still gives you keyboard controls.
+
+Then edit your [config](#configuration) to use this interface with portaudio:
+
+```
+method = portaudio
+source = "Soundflower (2ch)"
+```
 
 
 Running via ssh
@@ -327,6 +338,8 @@ exit with ctrl+z then run 'bg' to keep i
 
 You can also use Cava's output for other programs by using raw output mode, which will write bar data to `STDOUT` that can be piped into other processes. More information on this option is documented in [the example config file](/example_files/config).
 
+A useful starting point example script written in python that consumes raw data can be found [here](https://github.com/karlstav/cava/issues/123#issuecomment-307891020).
+
 Font notes
 ----------
 
